@@ -34,10 +34,8 @@ namespace xpyt
     XEUS_PYTHON_API void register_filename_mapping(const std::string& filename, int execution_count);
 
     XEUS_PYTHON_API XPYT_FORCE_PYBIND11_EXPORT
-    xerror extract_error(py::error_already_set& error);
-
-    XEUS_PYTHON_API XPYT_FORCE_PYBIND11_EXPORT
-    xerror extract_error(const py::object& type, const py::object& value, const py::object& traceback);
+    xerror extract_error(const py::list& error);
+    xerror extract_already_set_error(py::error_already_set& error);
 }
 
 #endif
