@@ -33,9 +33,6 @@ namespace xpyt
 
     XEUS_PYTHON_API bool holding_gil();
 
-    XEUS_PYTHON_API
-    std::string extract_parameter(std::string param, int argc, char* argv[]);
-
     XEUS_PYTHON_API
     bool extract_option(std::string short_opt, std::string long_opt, int argc, char* argv[]);
 
@@ -62,9 +59,6 @@ namespace xpyt
     XEUS_PYTHON_API
     void sigkill_handler(int sig);
 
-    XEUS_PYTHON_API
-    bool should_print_version(int argc, char* argv[]);
-
     XEUS_PYTHON_API
     void print_pythonhome();
 
