@@ -21,12 +21,13 @@
 #include <unistd.h>
 #endif
 
-#include "xeus/xeus_context.hpp"
 #include "xeus/xkernel.hpp"
 #include "xeus/xkernel_configuration.hpp"
 #include "xeus/xinterpreter.hpp"
+#include "xeus/xhelper.hpp"
 
-#include "xeus-zmq/xserver_shell_main.hpp"
+#include "xeus-zmq/xserver_zmq_split.hpp"
+#include "xeus-zmq/xzmq_context.hpp"
 
 #include "pybind11/embed.h"
 #include "pybind11/pybind11.h"
@@ -43,7 +44,7 @@ namespace py = pybind11;
 
 int main(int argc, char* argv[])
 {
-    if (xpyt::should_print_version(argc, argv))
+    if (xeus::should_print_version(argc, argv))
     {
         std::clog << "xpython " << XPYT_VERSION << std::endl;
         return 0;
@@ -69,22 +70,29 @@ int main(int argc, char* argv[])
 #endif
     signal(SIGINT, xpyt::sigkill_handler);
 
+    // Python initialization
+    PyStatus status;
+
+    PyConfig config;
+    PyConfig_InitPythonConfig(&config);
+    // config.isolated = 1;
+
     // Setting Program Name
     static const std::string executable(xpyt::get_python_path());
     static const std::wstring wexecutable(executable.cbegin(), executable.cend());
+    config.program_name = const_cast<wchar_t*>(wexecutable.c_str());
 
-    // On windows, sys.executable is not properly set with Py_SetProgramName
-    // Cf. https://bugs.python.org/issue34725
-    // A private undocumented API was added as a workaround in Python 3.7.2.
-    // _Py_SetProgramFullPath(const_cast<wchar_t*>(wexecutable.c_str()));
-    Py_SetProgramName(const_cast<wchar_t*>(wexecutable.c_str()));
-
-    // Setting PYTHONHOME
-    xpyt::set_pythonhome();
+    // Setting Python Home
+    static const std::string pythonhome{ xpyt::get_python_prefix() };
+    static const std::wstring wstr(pythonhome.cbegin(), pythonhome.cend());;
+    config.home = const_cast<wchar_t*>(wstr.c_str());
     xpyt::print_pythonhome();
 
-    // Instanciating the Python interpreter
-    py::scoped_interpreter guard;
+    // Implicitly pre-initialize Python
+    status = PyConfig_SetBytesArgv(&config, argc, argv);
+    if (PyStatus_Exception(status)) {
+        std::cerr << "Error:" << status.err_msg << std::endl;
+    }
 
     // Setting argv
     wchar_t** argw = new wchar_t*[size_t(argc)];
@@ -92,16 +100,23 @@ int main(int argc, char* argv[])
     {
         argw[i] = Py_DecodeLocale(argv[i], nullptr);
     }
-    PySys_SetArgvEx(argc, argw, 0);
+    PyWideStringList py_argw;
+    py_argw.length = argc;
+    py_argw.items = argw;
+
+    config.argv = py_argw;
+    config.parse_argv = 1;
+
     for(auto i = 0; i < argc; ++i)
     {
         PyMem_RawFree(argw[i]);
     }
     delete[] argw;
 
-    using context_type = xeus::xcontext_impl<zmq::context_t>;
-    using context_ptr = std::unique_ptr<context_type>;
-    context_ptr context = context_ptr(new context_type());
+    // Instantiating the Python interpreter
+    py::scoped_interpreter guard;
+
+    std::unique_ptr<xeus::xcontext> context = xeus::make_zmq_context();
 
     // Instantiating the xeus xinterpreter
     bool raw_mode = xpyt::extract_option("-r", "--raw", argc, argv);
@@ -119,7 +134,7 @@ int main(int argc, char* argv[])
     using history_manager_ptr = std::unique_ptr<xeus::xhistory_manager>;
     history_manager_ptr hist = xeus::make_in_memory_history_manager();
 
-    std::string connection_filename = xpyt::extract_parameter("-f", argc, argv);
+    std::string connection_filename = xeus::extract_filename(argc, argv);
 
 #ifdef XEUS_PYTHON_PYPI_WARNING
     std::clog <<
@@ -156,6 +171,7 @@ int main(int argc, char* argv[])
     }
     else
     {
+        std::clog << "Instantiating kernel" << std::endl;
         xeus::xkernel kernel(xeus::get_user_name(),
                              std::move(context),
                              std::move(interpreter),
@@ -165,6 +181,7 @@ int main(int argc, char* argv[])
                              xpyt::make_python_debugger,
                              debugger_config);
 
+        std::cout << "Getting config" << std::endl;
         const auto& config = kernel.get_config();
         std::clog <<
             "Starting xeus-python kernel...\n\n"
