@@ -42,7 +42,7 @@ using namespace std::placeholders;
 
 namespace xpyt
 {
-    debugger::debugger(zmq::context_t& context,
+    debugger::debugger(xeus::xcontext& context,
                        const xeus::xconfiguration& config,
                        const std::string& user_name,
                        const std::string& session_id,
@@ -195,7 +195,7 @@ namespace xpyt
     nl::json debugger::copy_to_globals_request(const nl::json& message)
     {
         // This request cannot be processed if the version of debugpy is lower than 1.6.5.
-        if (m_copy_to_globals_available)
+        if (!m_copy_to_globals_available)
         {
             nl::json reply = {
                 {"type", "response"},
@@ -300,12 +300,12 @@ namespace xpyt
             std::replace(version.begin(), version.end(), '.', ' ');
 
             // Check if the copy_to_globals feature is available
-            m_copy_to_globals_available = less_than_version(version, "1 6 5");
+            m_copy_to_globals_available = !less_than_version(version, "1 6 5");
         }
         return status == "ok";
     }
 
-    bool debugger::start(zmq::socket_t& header_socket, zmq::socket_t& request_socket)
+    bool debugger::start()
     {
         std::string temp_dir = xeus::get_temp_directory_path();
         std::string log_dir = temp_dir + "/" + "xpython_debug_logs_" + std::to_string(xeus::get_current_pid());
@@ -322,8 +322,7 @@ namespace xpyt
         std::string controller_header_end_point = xeus::get_controller_end_point("debugger_header");
         std::string publisher_end_point = xeus::get_publisher_end_point();
 
-        request_socket.bind(controller_end_point);
-        header_socket.bind(controller_header_end_point);
+        bind_sockets(controller_header_end_point, controller_end_point);
 
         std::string debugpy_end_point = "tcp://" + m_debugpy_host + ':' + m_debugpy_port;
         std::thread client(&xdap_tcp_client::start_debugger,
@@ -334,22 +333,19 @@ namespace xpyt
                            controller_header_end_point);
         client.detach();
 
-        request_socket.send(zmq::message_t("REQ", 3), zmq::send_flags::none);
-        zmq::message_t ack;
-        (void)request_socket.recv(ack);
+        send_recv_request("REQ");
 
-        std::string tmp_folder =  get_tmp_prefix();
+        std::string tmp_folder = get_tmp_prefix();
         xeus::create_directory(tmp_folder);
 
         return true;
     }
 
-    void debugger::stop(zmq::socket_t& header_socket, zmq::socket_t& request_socket)
+    void debugger::stop()
     {
         std::string controller_end_point = xeus::get_controller_end_point("debugger");
         std::string controller_header_end_point = xeus::get_controller_end_point("debugger_header");
-        request_socket.unbind(controller_end_point);
-        header_socket.unbind(controller_header_end_point);
+        unbind_sockets(controller_header_end_point, controller_end_point);
     }
 
     xeus::xdebugger_info debugger::get_debugger_info() const
@@ -358,7 +354,8 @@ namespace xpyt
                                     get_tmp_prefix(),
                                     get_tmp_suffix(),
                                     true,
-                                    {"Python Exceptions"});
+                                    {"Python Exceptions"},
+                                    true);
     }
 
     std::string debugger::get_cell_temporary_file(const std::string& code) const
@@ -372,7 +369,7 @@ namespace xpyt
                                                           const std::string& session_id,
                                                           const nl::json& debugger_config)
     {
-        return std::unique_ptr<xeus::xdebugger>(new debugger(context.get_wrapped_context<zmq::context_t>(),
+        return std::unique_ptr<xeus::xdebugger>(new debugger(context,
                                                              config, user_name, session_id, debugger_config));
     }
 }
