@@ -80,7 +80,7 @@ namespace xpyt
 
         instanciate_ipython_shell();
 
-        m_ipython_shell_app.attr("initialize")();
+        m_ipython_shell_app.attr("initialize")(use_jedi_for_completion());
         m_ipython_shell = m_ipython_shell_app.attr("shell");
 
         // Setting kernel property owning the CommManager and get_parent
@@ -109,14 +109,15 @@ namespace xpyt
         {
             redirect_output();
         }
+
+        py::module context_module = get_request_context_module();
     }
 
-    nl::json interpreter::execute_request_impl(int /*execution_count*/,
-                                               const std::string& code,
-                                               bool silent,
-                                               bool store_history,
-                                               nl::json user_expressions,
-                                               bool allow_stdin)
+    void interpreter::execute_request_impl(send_reply_callback cb,
+                                           int /*execution_count*/,
+                                           const std::string& code,
+                                           xeus::execute_request_config config,
+                                           nl::json user_expressions)
     {
         py::gil_scoped_acquire acquire;
         nl::json kernel_res;
@@ -126,14 +127,47 @@ namespace xpyt
 
         // Scope guard performing the temporary monkey patching of input and
         // getpass with a function sending input_request messages.
-        auto input_guard = input_redirection(allow_stdin);
+        auto input_guard = input_redirection(config.allow_stdin);
 
-        py::object ipython_res = m_ipython_shell.attr("run_cell")(code, "store_history"_a=store_history, "silent"_a=silent);
+        bool exception_occurred = false;
+        try
+        {
+            m_ipython_shell.attr("run_cell")(code, "store_history"_a=config.store_history, "silent"_a=config.silent);
+        }
+        catch(std::runtime_error& e)
+        {
+            const std::string error_msg = e.what();
+            if(!config.silent)
+            {
+                publish_execution_error("RuntimeError", error_msg, std::vector<std::string>());
+            }
+            kernel_res["ename"] = "std::runtime_error";
+            kernel_res["evalue"] = error_msg;
+            exception_occurred = true;
+        }
+        catch(...)
+        {
+            if(!config.silent)
+            {
+                publish_execution_error("unknown_error", "", std::vector<std::string>());
+            }
+            kernel_res["ename"] = "UnknownError";
+            kernel_res["evalue"] = "";
+            exception_occurred = true;
+        }
 
         // Get payload
         kernel_res["payload"] = m_ipython_shell.attr("payload_manager").attr("read_payload")();
         m_ipython_shell.attr("payload_manager").attr("clear_payload")();
 
+        if(exception_occurred)
+        {
+            kernel_res["status"] = "error";
+            kernel_res["traceback"] = std::vector<std::string>();
+            cb(kernel_res);
+            return;
+        }
+
         if (m_ipython_shell.attr("last_error").is_none())
         {
             kernel_res["status"] = "ok";
@@ -145,7 +179,7 @@ namespace xpyt
 
             xerror error = extract_error(pyerror);
 
-            if (!silent)
+            if (!config.silent)
             {
                 publish_execution_error(error.m_ename, error.m_evalue, error.m_traceback);
             }
@@ -155,8 +189,7 @@ namespace xpyt
             kernel_res["evalue"] = error.m_evalue;
             kernel_res["traceback"] = error.m_traceback;
         }
-
-        return kernel_res;
+        cb(kernel_res);
     }
 
     nl::json interpreter::complete_request_impl(
@@ -296,7 +329,6 @@ namespace xpyt
         try
         {
             exec(py::str(code));
-
             reply["status"] = "ok";
         }
         catch (py::error_already_set& e)
@@ -321,6 +353,40 @@ namespace xpyt
         return reply;
     }
 
+    void interpreter::set_request_context(xeus::xrequest_context context)
+    {
+        py::gil_scoped_acquire acquire;
+        py::module context_module = get_request_context_module();
+        context_module.attr("set_request_context")(context);
+    }
+
+    namespace
+    {
+        xeus::xrequest_context empty_request_context{};
+    }
+
+    const xeus::xrequest_context& interpreter::get_request_context() const noexcept
+    {
+        py::gil_scoped_acquire acquire;
+        py::module context_module = get_request_context_module();
+        // When the debugger is started, it send some python code to execute, that triggers
+        // a call to publish_stream, and ultimately to this function. However:
+        // - we are out of the handling of an execute_request, therefore set_request_context
+        // has not been called
+        // - we cannot set it from another thread (the context of the context variable would
+        // be different)
+        // Therefore, we have to catch the exception thrown when the context variable is empty.
+        try
+        {
+            py::object res = context_module.attr("get_request_context")();
+            return *(res.cast<xeus::xrequest_context*>());
+        }
+        catch (py::error_already_set& e)
+        {
+            return empty_request_context;
+        }
+    }
+
     void interpreter::redirect_output()
     {
         py::module sys = py::module::import("sys");
@@ -335,4 +401,8 @@ namespace xpyt
         m_ipython_shell_app = py::module::import("xeus_python_shell.shell").attr("XPythonShellApp")();
     }
 
+    bool interpreter::use_jedi_for_completion() const
+    {
+        return true;
+    }
 }
