@@ -39,11 +39,8 @@
 #include "pybind11/pybind11.h"
 #include "pybind11/eval.h"
 
-#include "xtl/xhash.hpp"
-
 #include "xeus-python/xutils.hpp"
 
-
 namespace py = pybind11;
 namespace nl = nlohmann;
 
@@ -69,25 +66,6 @@ namespace xpyt
         return py::eval(code, scope);
     }
 
-    std::string extract_parameter(std::string param, int argc, char* argv[])
-    {
-        std::string res = "";
-        for (int i = 0; i < argc; ++i)
-        {
-            if ((std::string(argv[i]) == param) && (i + 1 < argc))
-            {
-                res = argv[i + 1];
-                for (int j = i; j < argc - 2; ++j)
-                {
-                    argv[j] = argv[j + 2];
-                }
-                argc -= 2;
-                break;
-            }
-        }
-        return res;
-    }
-
     bool extract_option(std::string short_opt, std::string long_opt, int argc, char* argv[])
     {
         bool res = false;
@@ -113,7 +91,7 @@ namespace xpyt
         void* array[10];
 
         // get void*'s for all entries on the stack
-        size_t size = backtrace(array, 10);
+        int size = backtrace(array, 10);
 
         // print out all the frames to stderr
         fprintf(stderr, "Error: signal %d:\n", sig);
@@ -127,27 +105,22 @@ namespace xpyt
         exit(0);
     }
 
-    bool should_print_version(int argc, char* argv[])
-    {
-        for (int i = 0; i < argc; ++i)
-        {
-            if (std::string(argv[i]) == "--version")
-            {
-                return true;
-            }
-        }
-        return false;
-    }
-
     void print_pythonhome()
     {
         std::setlocale(LC_ALL, "en_US.utf8");
+        // Py_GetPythonHome will return NULL if called before Py_Initialize()
         wchar_t* ph = Py_GetPythonHome();
 
-        char mbstr[1024];
-        std::wcstombs(mbstr, ph, 1024);
-
-        std::clog << "PYTHONHOME set to " << mbstr << std::endl;
+        if (ph)
+        {
+            char mbstr[1024];
+            std::wcstombs(mbstr, ph, 1024);
+            std::clog << "PYTHONHOME set to " << mbstr << std::endl;
+        }
+        else
+        {
+            std::clog << "PYTHONHOME not set or not initialized." << std::endl;
+        }
     }
 
     // Compares 2 versions and return true if version1 < version2.
